function Ts_yx = bubbleDew(zis, Tmax, step, P, props, maxiter, convTtol, dT)
% Returns a 4xlength(zis) matrix
%            split = [ Tbub  0     0    ...  0           [K] bubble point
%                      Tdew  0     0    ...  0           [K] dew point
%                      y1    y2    y3   ...  yn          [mol.frac] vapor phase
%                      x1    x2    x3   ...  xn  ]       [mol.frac] liquid phase
% ========================================================================= Specifications
alphafun = "SRK"    ;
mixrule  = "Virial" ;
Nel = length(zis);
% =========================================================================== Calculations
fprintf( "\n========================== DEWT\n" ) ;
yis      = zis    ;                                     % [mol.frac]
flagsup = 0 ;
for( T = Tmax: -step : 0 )

	Kis      = Wilson_correl_Ks( T , P , props ) ;         % []
	xis      = yis./Kis                          ;         % [mol.frac]
	xis	     = xis/sum( xis )                    ;         % [mol.frac] normalize

	params = SRK_mix_parameters_kij( T , P , xis , props , alphafun, mixrule ) ;
	Zs     = SRK_compress_factors( params )                                    ; % []
	phisl  = SRK_parc_fug_coef_all( xis , params , Zs( 2 ) )                   ; % []
	
	params = SRK_mix_parameters_kij( T , P , yis , props , alphafun, mixrule ) ;
	Zs     = SRK_compress_factors( params )                                    ; % []
	phisv  = SRK_parc_fug_coef_all( yis , params , Zs( 1 ) )                   ; % []
	
	if( iscomplex( phisl./phisv ) )
		if(  flagsup == 0 )
			continue ;
		else
			Tinf = T ;
			break    ;
		end
	else
		if( flagsup == 0 )
			Tsup = T    ;
			flagsup = 1 ;
		end
	end

end
Tsup                                                       % [K] 
Tinf                                                       % [K] 

Tinit    = ( Tsup + Tinf )/2                               % [K]
T        = Tinit                                           % [K]
for( k = 1 : maxiter )

	% ----------------------------------- Newton-Raphson iteration
	OFs = [ ] ;                                            % [] then [OF] then [OF incOF]
	for( T = [ T  ( T + dT ) ] )                           % 2 rounds, OF and incOF

		% function (OF) body >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
		params = SRK_mix_parameters_kij( T , P , xis , props , alphafun, mixrule ) ;
		Zs     = SRK_compress_factors( params )                                    ; % []
		phisl  = SRK_parc_fug_coef_all( xis , params , Zs( 2 ) )                   ; % []
		
		params = SRK_mix_parameters_kij( T , P , yis , props , alphafun, mixrule ) ;
		Zs     = SRK_compress_factors( params )                                    ; % []
		phisv  = SRK_parc_fug_coef_all( yis , params , Zs( 1 ) )                   ; % []

		Kis    = phisl./phisv        ;                     % []

		xisT   = yis./Kis            ;                     % [mol.frac]
		OF     = sum( yis./Kis ) - 1 ;                     % [mol.frac]
		% <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< function (OF) body
	  
		OFs   = [ OFs OF ]           ;                     % [OF] then [OF incOF]
		
	end
	T  = T - dT - OFs( 1 )/( OFs( 2 ) - OFs( 1 ) )*dT ;    % [K]
	if( abs( OFs( 1 ) ) < convTtol )
		break ;
	end
	xis      = xisT ;                                      % [mol.frac]

end
% ----------------------------------------------------------------------- Checks
assert( k < maxiter , "\n----- Error: max. iterations reached" ) ;
check = sum( abs( xis.*phisl - yis.*phisv ) )                    ;
if( abs( check ) > 1e-5 )
	fprintf( 'Check failed: sum( abs( xis.*phisl - yis.*phisv ) ) = %6.2g > 1e-5\n\n', ...
		check ) ; %return ;
end
xdews   = xis                      ;                      % [mol.frac]
check_xis_1 = sum( xis )
Tdew = T                            ;                      % [K]
fprintf( "Tdew = %12.6f\n" , Tdew ) ;


% =================================================================================== BUBT
fprintf( "\n========================== BUBT\n" ) ;

xis      = zis ;                                         % [mol.frac]
flagsup = 0      ;                                         % flag complex Zs upper limit
for( T = Tdew: -step : 0 )

	Kis      = Wilson_correl_Ks( T , P , props ) ;         % []
	yis      = Kis.*xis                          ;         % [mol.frac]
	yis      = yis/sum( yis )                    ;         % [mol.frac] normalize

	params = SRK_mix_parameters_kij( T , P , xis , props , alphafun, mixrule ) ;
	Zs     = SRK_compress_factors( params )                                    ; % []
	phisl  = SRK_parc_fug_coef_all( xis , params , Zs( 2 ) )                   ; % []
	
	params = SRK_mix_parameters_kij( T , P , yis , props , alphafun, mixrule ) ;
	Zs     = SRK_compress_factors( params )                                    ; % []
	phisv  = SRK_parc_fug_coef_all( yis , params , Zs( 1 ) )                   ; % []
	
	if( iscomplex( phisl./phisv ) )
		if(  flagsup == 0 )
			continue ;
		else
			Tinf = T ;
			break    ;
		end
	else
		if( flagsup == 0 )
			Tsup = T    ;
			flagsup = 1 ;
		end
	end

end
Tsup                                                       % [K] 
Tinf                                                       % [K] 

xis      = zis              ;                            % [mol.frac]
for( T = Tinf : step : Tsup ) ;
	% T % follow

	Kis      = Wilson_correl_Ks( T , P , props ) ;         % []
	yis      = Kis.*xis                          ;         % [mol.frac]
	yis      = yis/sum( yis )                    ;         % [mol.frac] normalize

	flagnext = 0 ;                                         % flag: next T 
	flagconv = 0 ;                                         % flag: converged 

	for( k = 1 : maxiter )

		% ----------------------------------- Newton-Raphson iteration
		OFs = [ ] ;                                        % [] then [OF] then [OF incOF]
		for( T = [ T  ( T + dT ) ] )                       % 2 rounds, OF and incOF

			% function (OF) body >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
			params = SRK_mix_parameters_kij( T , P , xis , props , alphafun, mixrule ) ;
			Zs     = SRK_compress_factors( params )                                 ; % []
			phisl  = SRK_parc_fug_coef_all( xis , params , Zs( 2 ) )                ; % []
			
			if( iscomplex( phisl ) )
				flagnext = 1 ;
			end
			
			params = SRK_mix_parameters_kij( T , P , yis , props , alphafun, mixrule ) ;
			Zs     = SRK_compress_factors( params )                                 ; % []
			phisv  = SRK_parc_fug_coef_all( yis , params , Zs( 1 ) )                ; % []

			if( iscomplex( phisv ) )
				flagnext = 1 ;
			end

			Kis    = phisl./phisv ;                                % []

			yisT   = Kis.*xis            ;                         % [mol.frac]
			OF     = sum( xis.*Kis ) - 1 ;                         % [mol.frac]
			% <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< function (OF) body
		  
			OFs   = [ OFs OF ]      ;                          % [OF] then [OF incOF]
			
		end
		if( flagnext == 1 )
			break ;
		end
		
		T  = T - dT - OFs( 1 )/( OFs( 2 ) - OFs( 1 ) )*dT ; % [K]
		if( abs( OFs( 1 ) ) < convTtol )
			flagconv = 1 ;
			break ;
		end
		yis      = yisT ;                                  % [mol.frac]
	
	end

	if( flagconv == 1 )
		break ;
	end
	if( flagnext == 1 )
		continue ;
	end

end
% ----------------------------------------------------------------------- Checks
assert( T < Tsup , "\n----- Error: BUBT not found in the whole range" ) ;
check = sum( abs( xis.*phisl - yis.*phisv ) ) ;
if( abs( check ) > 1e-5 )
	fprintf( 'Check failed: sum( abs( xis.*phisl - yis.*phisv ) ) = %6.2g > 1e-5\n\n' , ...
		check ) ; %return ;
end
assert( k < maxiter , "\n----- Error: max. iterations reached" ) ;
check_yis_1 = sum( yis )
Tbub     = T                          ;                    % [K]
fprintf( "Tbub = %12.6f\n" , Tbub )   ;
ybubs   = yis                         ;                    % [mol.frac.]
fprintf( "Tdew = %12.6f\n" , Tdew )   ;
xdews                                 ;                    % [mol.frac.]
zis                                   ;                    % [mol.frac.]
P                                     ;                    % [bar] 
Ts_yx=[Tbub zeros(1,Nel-1); Tdew zeros(1,Nel-1); ybubs; xdews];
fprintf('\n\n') ;
end