function TvsHs = flashEnthalpy(zis, Tmax, step, Tin, Tout, P, props, maxiter, convTtol, Nst, dT, Hfun)
	% ========================================================================= Specifications
	alphafun = "SRK"    ;
	mixrule  = "Virial" ;
	Nel = length(zis);
	% =========================================================================== Calculations
	fprintf( "\n========================== DEWT\n" ) ;
	yis      = zis    ;                                     % [mol.frac]
	flagsup = 0 ;
	for( T = Tmax: -step : 0 )
		
		Kis      = Wilson_correl_Ks( T , P , props ) ;         % []
		xis      = yis./Kis                          ;         % [mol.frac]
		xis	     = xis/sum( xis )                    ;         % [mol.frac] normalize

		params = SRK_mix_parameters_kij( T , P , xis , props , alphafun, mixrule ) ;
		Zs     = SRK_compress_factors( params )                                    ; % []
		phisl  = SRK_parc_fug_coef_all( xis , params , Zs( 2 ) )                   ; % []
		
		params = SRK_mix_parameters_kij( T , P , yis , props , alphafun, mixrule ) ;
		Zs     = SRK_compress_factors( params )                                    ; % []
		phisv  = SRK_parc_fug_coef_all( yis , params , Zs( 1 ) )                   ; % []

		if( iscomplex( phisl./phisv ) )
			if(  flagsup == 0 )
				continue ;
			else
				Tinf = T ;
				break    ;
			end
		else
			if( flagsup == 0 )
				Tsup = T    ;
				flagsup = 1 ;
			end
		end

	end
	Tsup                                                       % [K] 
	Tinf                                                       % [K] 

	Tinit    = ( Tsup + Tinf )/2                               % [K]
	T        = Tinit                                           % [K]
	for( k = 1 : maxiter )

		% ----------------------------------- Newton-Raphson iteration
		OFs = [ ] ;                                            % [] then [OF] then [OF incOF]
		for( T = [ T  ( T + dT ) ] )                           % 2 rounds, OF and incOF

			% function (OF) body >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
			params = SRK_mix_parameters_kij( T , P , xis , props , alphafun, mixrule ) ;
			Zs     = SRK_compress_factors( params )                                    ; % []
			phisl  = SRK_parc_fug_coef_all( xis , params , Zs( 2 ) )                   ; % []
			
			params = SRK_mix_parameters_kij( T , P , yis , props , alphafun, mixrule ) ;
			Zs     = SRK_compress_factors( params )                                    ; % []
			phisv  = SRK_parc_fug_coef_all( yis , params , Zs( 1 ) )                   ; % []

			Kis    = phisl./phisv        ;                     % []

			xisT   = yis./Kis            ;                     % [mol.frac]
			OF     = sum( yis./Kis ) - 1 ;                     % [mol.frac]
			% <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< function (OF) body
		  
			OFs   = [ OFs OF ]           ;                     % [OF] then [OF incOF]
			
		end
		T  = T - dT - OFs( 1 )/( OFs( 2 ) - OFs( 1 ) )*dT ;    % [K]
		if( abs( OFs( 1 ) ) < convTtol )
			break ;
		end
		xis      = xisT ;                                      % [mol.frac]

	end
	% ----------------------------------------------------------------------- Checks
	assert( k < maxiter , "\n----- Error: max. iterations reached" ) ;
	check = sum( abs( xis.*phisl - yis.*phisv ) )                    ;
	if( abs( check ) > 1e-5 )
		fprintf( 'Check failed: sum( abs( xis.*phisl - yis.*phisv ) ) = %6.2g > 1e-5\n\n', ...
			check ) ; %return ;
	end
	xdews   = xis                      ;                      % [mol.frac]
	check_xis_1 = sum( xis )
	Tdew = T                            ;                      % [K]
	fprintf( "Tdew = %12.6f\n" , Tdew ) ;


	% =================================================================================== BUBT
	fprintf( "\n========================== BUBT\n" ) ;

	xis      = zis ;                                         % [mol.frac]
	flagsup = 0      ;                                         % flag complex Zs upper limit
	for( T = Tdew: -step : 0 )

		Kis      = Wilson_correl_Ks( T , P , props ) ;         % []
		yis      = Kis.*xis                          ;         % [mol.frac]
		yis      = yis/sum( yis )                    ;         % [mol.frac] normalize

		params = SRK_mix_parameters_kij( T , P , xis , props , alphafun, mixrule ) ;
		Zs     = SRK_compress_factors( params )                                    ; % []
		phisl  = SRK_parc_fug_coef_all( xis , params , Zs( 2 ) )                   ; % []
		
		params = SRK_mix_parameters_kij( T , P , yis , props , alphafun, mixrule ) ;
		Zs     = SRK_compress_factors( params )                                    ; % []
		phisv  = SRK_parc_fug_coef_all( yis , params , Zs( 1 ) )                   ; % []
		
		if( iscomplex( phisl./phisv ) )
			if(  flagsup == 0 )
				continue ;
			else
				Tinf = T ;
				break    ;
			end
		else
			if( flagsup == 0 )
				Tsup = T    ;
				flagsup = 1 ;
			end
		end

	end
	Tsup                                                       % [K] 
	Tinf                                                       % [K] 

	xis      = zis              ;                            % [mol.frac]
	for( T = Tinf : step : Tsup ) ;
		% T % follow

		Kis      = Wilson_correl_Ks( T , P , props ) ;         % []
		yis      = Kis.*xis                          ;         % [mol.frac]
		yis      = yis/sum( yis )                    ;         % [mol.frac] normalize

		flagnext = 0 ;                                         % flag: next T 
		flagconv = 0 ;                                         % flag: converged 
		for( k = 1 : maxiter )

			% ----------------------------------- Newton-Raphson iteration
			OFs = [ ] ;                                        % [] then [OF] then [OF incOF]
			for( T = [ T  ( T + dT ) ] )                       % 2 rounds, OF and incOF

				% function (OF) body >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
				params = SRK_mix_parameters_kij( T , P , xis , props , alphafun, mixrule ) ;
				Zs     = SRK_compress_factors( params )                                 ; % []
				phisl  = SRK_parc_fug_coef_all( xis , params , Zs( 2 ) )                ; % []
				
				if( iscomplex( phisl ) )
					flagnext = 1 ;
				end
				
				params = SRK_mix_parameters_kij( T , P , yis , props , alphafun, mixrule ) ;
				Zs     = SRK_compress_factors( params )                                 ; % []
				phisv  = SRK_parc_fug_coef_all( yis , params , Zs( 1 ) )                ; % []

				if( iscomplex( phisv ) )
					flagnext = 1 ;
				end

				Kis    = phisl./phisv ;                                % []

				yisT   = Kis.*xis            ;                         % [mol.frac]
				OF     = sum( xis.*Kis ) - 1 ;                         % [mol.frac]
				% <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< function (OF) body
			  
				OFs   = [ OFs OF ]      ;                          % [OF] then [OF incOF]
				
			end
			if( flagnext == 1 )
				break ;
			end
			
			T  = T - dT - OFs( 1 )/( OFs( 2 ) - OFs( 1 ) )*dT ; % [K]
			if( abs( OFs( 1 ) ) < convTtol )
				flagconv = 1 ;
				break ;
			end
			yis      = yisT ;                                  % [mol.frac]
		
		end
		if( flagconv == 1 )
			break ;
		end
		if( flagnext == 1 )
			continue ;
		end

	end
	% ----------------------------------------------------------------------- Checks
	assert( T < Tsup , "\n----- Error: BUBT not found in the whole range" ) ;
	check = sum( abs( xis.*phisl - yis.*phisv ) ) ;
	if( abs( check ) > 1e-5 )
		fprintf( 'Check failed: sum( abs( xis.*phisl - yis.*phisv ) ) = %6.2g > 1e-5\n\n' , ...
			check ) ; %return ;
	end
	assert( k < maxiter , "\n----- Error: max. iterations reached" ) ;
	check_yis_1 = sum( yis )
	Tbub     = T                          ;                    % [K]
	fprintf( "Tbub = %12.6f\n" , Tbub )   ;
	ybubs   = yis                         ;                    % [mol.frac.]
	fprintf( "Tdew = %12.6f\n" , Tdew )   ;
	xdews                                 ;                    % [mol.frac.]
	zis                                   ;                    % [mol.frac.]
	P                                     ;                    % [bar] 
	fprintf('\n\n') ;

	% There are a bunch of fringe cases left to add yet. Only when 2 phases are presents is kinda ok.
	Thigh=max(Tin,Tout);
	Tlow=min(Tin,Tout);
	Trange = Thigh-Tlow;
	if(Tlow>=Tdew)
		nptsv=Nst;
	elseif(Thigh>=Tdew&&Tlow<Tdew&&Tlow>Tbub)
		nptsv=max(min(ceil(Nst*(Thigh-Tdew)/Trange),Nst-1),1);
		nptslv=Nst-nptsv;
	elseif(Thigh>=Tbub&&Tlow<=Tdew) % maybe forgetting something for pure or almost pure cases
		nptsv=max(ceil(Nst*(Thigh-Tdew)/Trange)-1,1);
		nptsl=max(ceil(Nst*(Tbub-Tlow)/Trange)-1,1);
		nptslv=Nst-nptsv-nptsl-2;
	elseif(Thigh<Tdew&&Tlow>Tbub)
		nptslv=Nst;
	elseif(Thigh<Tdew&&Thigh>Tbub&&Tlow<=Tbub)
		nptslv=max(ceil(Nst*(Thigh-Tbub)/Trange)-1,1);
		nptsl=Nst-nptsl
	else
		nptsl=Nst;
	end

	% =========================================================================== Calculations

	TvsHsv   = [ ]   ;
	TvsHsl   = [ ]   ;
	TvsHslv  = [ ]   ;
	
	yis      = zis ;                                         % [mol.frac.]

	params   = SRK_mix_parameters_kij( Thigh , P , yis , props , alphafun, mixrule ) ; 
	Zs       = SRK_compress_factors( params )                                     ;
	Hres_high    = SRK_Hres( Thigh , params , Zs(1) ) ;               % [J/mol]
	
	for(k=1:Nel)
		AL_Hid_Thigh(k)=Hfun{k}(Thigh);
	end
	
	if(Thigh>=Tdew)
		%fprintf( "\n========================== Thigh => DEWT\n" ) ;
		Tmin = max(Tdew,Tlow);
		incT     = ( Tmin - Thigh )/nptsv       ;
		for( T = Thigh : incT : Tmin )

			params   = SRK_mix_parameters_kij( T , P , yis , props , alphafun, mixrule ) ;
			Zs       = SRK_compress_factors( params )                                    ;
			Hres     = SRK_Hres( T , params , Zs(1) )      ;       % [J/mol]

			dHid = 0;
			for(k=1:Nel)
				dHid=dHid+zis(k)*(Hfun{k}(T)-AL_Hid_Thigh(k));
			end
			
			dHmol    = dHid + Hres_high - Hres                 ;       % [J/mol]
			TvsHsv   = [ TvsHsv ; [ T dHmol ] ]            ;

		end
	end
	
	if(Tlow<=Tbub)
		%fprintf( "\n========================== BUBT => Tlow\n" ) ;
		xis      = zis               ;                           % [mol.frac.]
		Tmax = min(Thigh,Tbub);
		incT     = ( Tlow - Tmax )/nptsl ;                           % [K]
		for( T = Tmax : incT : Tlow )

			params   = SRK_mix_parameters_kij( T , P , xis , props , alphafun, mixrule ) ;
			Zs       = SRK_compress_factors( params )                                    ;
			Hres     = SRK_Hres( T , params , Zs(2) )      ;       % [J/mol]

			dHid = 0;
			for(k=1:Nel)
				dHid=dHid+zis(k)*(Hfun{k}(T)-AL_Hid_Thigh(k));
			end
			
			dHmol    = dHid + Hres_high - Hres                 ;       % [J/mol]
			TvsHsl   = [ TvsHsl ; [ T dHmol ] ]            ;
			
		end
	end
	
	if(Thigh>Tbub&&Tlow<Tdew&&nptslv>0)
		%fprintf( "\n========================== DEWT => BUBT\n" ) ;
		bV       = 1        ;                                      % [] vapor fraction
		yis      = zis      ;                                      % [mol.frac.]
		xis      = xdews    ;                                      % [mol.frac.]
		Kis      = yis./xis ;                                      % [ ] init
		prevKis  = Kis      ;                                      % []
		
		% Conditions pending
		Tmin = max(Tlow,Tbub);
		Tmax = min(Thigh,Tdew);
		incT = ( Tmin - Tmax )/(nptslv+1) ;
		for( T = Tdew + incT : incT : Tbub - incT )
			
			for( k = 1 : maxiter )
				
				for( j = 1 : maxiter )

					% ----------------------------------- Newton-Raphson iteration
					OFs = [ ] ;                                    % [] then [OF] then [OF incOF]
					for( bV = [ bV  ( bV + dT ) ] )                % 2 rounds, OF and incOF

						OF = sum( zis.*( Kis - 1 )./( 1 + bV*( Kis - 1 ) ) ) ; % [] Obj. Func.
						OFs   = [ OFs OF ] ;                       % [OF] then [OF incOF]
						
					end
					bV  = bV - dT - OFs( 1 )/( OFs( 2 ) - OFs( 1 ) )*dT ; % [K]
					if( abs( OFs( 1 ) ) < convTtol )
						break ;
					end

				end
				assert( j < maxiter , "\n----- Error: max. iterations reached" ) ;
				
				xis      = zis./( 1 + bV*( Kis - 1 ) ) ;         % [mol.frac]
				yis      = xis.*Kis                      ;         % [mol.frac]

				% ---------------------------------------------------- Soave-improving Kis
				paramsl = SRK_mix_parameters_kij( T , P , xis , props , 'SRK', 'Virial') ;
				Zl      = SRK_compress_factors( paramsl )(2)          ; % []
				phisl   = SRK_parc_fug_coef_all( xis , paramsl , Zl ) ; % []
				
				paramsv = SRK_mix_parameters_kij( T , P , yis , props , 'SRK', 'Virial') ;
				Zv      = SRK_compress_factors( paramsv )(1)          ; % []
				phisv   = SRK_parc_fug_coef_all( yis , paramsv , Zv ) ; % []
				
				fugsv  = phisv.*yis   ;                            % []
				fugsl  = phisl.*xis   ;                            % []
				Kis    = phisl./phisv ;                            % []

				errKis = sum( abs( Kis - prevKis ) ) ;             % []
				if( errKis < convTtol )
					break ;
				end
				prevKis = Kis                        ;             % []
			end
			assert( k < maxiter , "Max. number of iterations reached (state flash)" ) ;
			assert( abs( sum(xis) - 1 ) < 1e-8 , "sum(xis) - 1 > 1e-8" )              ; 
			assert( abs( sum(yis) - 1 ) < 1e-8 , "sum(yis) - 1 > 1e-8" )              ; 
			errFug = sum( ( fugsl./fugsv .- 1 ).^2 )                                  ;
			assert( errFug < 1e-12 , "sum( ( fugsl./fugsv .- 1 ).^2 )  > 1e-12" )     ;      

			Hresl    = SRK_Hres( T , paramsl , Zl )        ;       % [J/mol]
			Hresv    = SRK_Hres( T , paramsv , Zv )        ;       % [J/mol]
			Hres     = ( 1 - bV )*Hresl + bV*Hresv         ;       % [J/mol]
			
			dHid = 0;
			for(k=1:Nel)
				dHid=dHid+zis(k)*(Hfun{k}(T)-AL_Hid_Thigh(k));
			end
			
			dHmol    = dHid + Hres_high - Hres ;                       % [J/mol]
			TvsHslv  = [ TvsHslv ; [ T dHmol ] ] ;

		end
	end
	
	TvsHs    = [ TvsHsv ; TvsHslv ; TvsHsl ] ;


%{
	figure( 1 ) ;
	clf ;
	t = title( 'NG COOLING' ,'fontsize', 20 ) ;
	%set ( t , "fontsize" , 20 );
	line( TvsHs( : , 2 ) , TvsHs( : , 1 ) ,...
		'Color' , 'black' ,'Marker' , 'o' ) ;
	xlabel('Enthalpy change (J/mol)' ,'fontsize', 20)
	ylabel('T (K)' , 'fontsize' , 20)
	xlim( [ TvsHs(end,2) TvsHs(1,2) ] ) ;
	%grid on ;

	fprintf( "\n========================== Thigh => DEWT\n" ) ;
	fprintf( "   T(K)      dH(J/mol)\n" ) ;
	fprintf( "-----------------------\n" ) ;

	%length( TvsHsv( k , : ) )
	for( k = 1 : nptsv + 1)
		fprintf( "%8.5f %12.3f\n" , TvsHsv( k , : ) ) ;
	end

	fprintf( "========================== DEWT => BUBT\n" ) ;
	for( k = 1 : nptslv )
		fprintf( "%8.5f %12.3f\n" , TvsHslv( k , : ) ) ;
	end


	fprintf( "========================== DEWT => Tlow\n" ) ;
	for( k = 1 : nptsl + 1)
		fprintf( "%8.5f %12.3f\n" , TvsHsl( k , : ) ) ;
	end

	fprintf( "\n\n========================== Thigh => DEWT\n" ) ;
	fprintf( "   T(K)      dH(J/mol)\n" ) ;
	fprintf( "-----------------------\n" ) ;
	for( k = 1 : nptsv + nptslv + nptsl + 2 )
		fprintf( "%8.5f %12.3f\n" , TvsHs( k , : ) ) ;
	end

	fprintf('\n\n') ;
%}
end
