function THpoint = pointEnthalpy(zis, Tbub, Tdew, ybubs, xdews, Tp, P, props, maxiter, convTtol, dT, Hfun)
	% ========================================================================= Specifications
	alphafun = "SRK"    ;
	mixrule  = "Virial" ;
	Nel=length(zis);
	% =========================================================================== Calculations
	if(Tp>=Tdew)
		%fprintf( "\n========================== T > DEWT\n" ) ;
		yis      = zis               ;                           % [mol.frac.]
		params   = SRK_mix_parameters_kij( Tp , P , yis , props , alphafun, mixrule ) ;
		Zs       = SRK_compress_factors( params )                                    ;
		Hres     = SRK_Hres( Tp , params , Zs(1) )      ;       % [J/mol]

		dHid = 0;
		for(k=1:Nel)
			dHid=dHid+zis(k)*Hfun{k}(Tp);
		end
		
		dHmol    = dHid - Hres                 ;       % [J/mol]

	elseif(Tp<=Tbub)
		%fprintf( "\n========================== BUBT => T\n" ) ;
		xis      = zis               ;                           % [mol.frac.]
		params   = SRK_mix_parameters_kij( Tp , P , xis , props , alphafun, mixrule ) ;
		Zs       = SRK_compress_factors( params )                                    ;
		Hres     = SRK_Hres( Tp , params , Zs(2) )      ;       % [J/mol]

		dHid = 0;
		for(k=1:Nel)
			dHid=dHid+zis(k)*Hfun{k}(Tp);
		end
		
		dHmol    = dHid - Hres                 ;       % [J/mol]

	else
		%fprintf( "\n========================== DEWT => BUBT\n" ) ;
		bV       = 1        ;                                      % [] vapor fraction
		yis      = zis      ;                                      % [mol.frac.]
		xis      = xdews    ;                                      % [mol.frac.]
		Kis      = yis./xis ;                                      % [ ] init
		prevKis  = Kis      ;                                      % []
		nptslv=4; % Increase the number in case this is not working. 3 seems the minimum, in some cases need more than 20
		incT=(Tp-Tdew)/nptslv;
		for( T = Tdew + incT : incT : Tp ) % 
			
			for( k = 1 : maxiter )
				
				for( j = 1 : maxiter )
					
					% ----------------------------------- Newton-Raphson iteration
					OFs = [ ] ;                                    % [] then [OF] then [OF incOF]
					for( bV = [ bV  ( bV + dT ) ] )                % 2 rounds, OF and incOF

						OF = sum( zis.*( Kis - 1 )./( 1 + bV*( Kis - 1 ) ) ) ; % [] Obj. Func.
						OFs   = [ OFs OF ]          ;              % [OF] then [OF incOF]
						
					end
					bV  = bV - dT - OFs( 1 )/( OFs( 2 ) - OFs( 1 ) )*dT ; % [K]
					if( abs( OFs( 1 ) ) < convTtol )
						break ;
					end

				end
				assert( j < maxiter , "\n----- Error: max. iterations reached" ) ;
				
				xis      = zis./( 1 + bV*( Kis - 1 ) )   ;         % [mol.frac]
				yis      = xis.*Kis                      ;         % [mol.frac]

				% ---------------------------------------------------- Soave-improving Kis
				paramsl = SRK_mix_parameters_kij( T , P , xis , props , 'SRK', 'Virial') ;
				Zl      = SRK_compress_factors( paramsl )(2)          ; % []
				phisl   = SRK_parc_fug_coef_all( xis , paramsl , Zl ) ; % []
				
				paramsv = SRK_mix_parameters_kij( T , P , yis , props , 'SRK', 'Virial') ;
				Zv      = SRK_compress_factors( paramsv )(1)          ; % []
				phisv   = SRK_parc_fug_coef_all( yis , paramsv , Zv ) ; % []
				
				fugsv  = phisv.*yis   ;                            % []
				fugsl  = phisl.*xis   ;                            % []
				Kis    = phisl./phisv ;                            % []

				errKis = sum( abs( Kis - prevKis ) )      ;        % []
				if( errKis < convTtol )
					break ;
				end
				prevKis = Kis                        ;             % []
			end
			assert( k < maxiter , "Max. number of iterations reached (state flash)" ) ;
			assert( abs( sum(xis) - 1 ) < 1e-8 , "sum(xis) - 1 > 1e-8" )              ; 
			assert( abs( sum(yis) - 1 ) < 1e-8 , "sum(yis) - 1 > 1e-8" )              ; 
			errFug = sum( ( fugsl./fugsv .- 1 ).^2 )                                  ;
			assert( errFug < 1e-12 , "sum( ( fugsl./fugsv .- 1 ).^2 )  > 1e-12" )     ;      
		end
		Hresl    = SRK_Hres( T , paramsl , Zl )        ;       % [J/mol]
		Hresv    = SRK_Hres( T , paramsv , Zv )        ;       % [J/mol]
		Hres     = ( 1 - bV )*Hresl + bV*Hresv         ;       % [J/mol]

		dHid = 0;
		for(k=1:Nel)
			dHid=dHid+zis(k)*Hfun{k}(T);
		end
		
		dHmol    = dHid - Hres ;                       % [J/mol]
		
	end
	
	THpoint    = [ Tp dHmol ] ;
	
end