function TSpoint = pointEntropy(zis, Tbub, Tdew, ybubs, xdews, Tp, P, props, maxiter, convTtol, dT, Sfun)
% ========================================================================= Specifications
alphafun = "SRK"    ;
mixrule  = "Virial" ;
Nel=length(zis);
R=8.3144598;
% =========================================================================== Calculations
if(Tp>=Tdew)
	%fprintf( "\n========================== T > DEWT\n" ) ;
	yis      = zis               ;                           % [mol.frac.]
	params   = SRK_mix_parameters_kij( Tp , P , yis , props , alphafun, mixrule ) ;
	Zs       = SRK_compress_factors( params )                                    ;
	Sres     = SRK_Sres( Tp , params , Zs(1) )      ;       % [J/mol]

	dSid = 0;
	for(k=1:Nel)
		dSid=dSid+zis(k)*Sfun{k}(Tp);
	end
	
	dSmol    = dSid - R*log(P) - Sres                 ;       % [J/mol]
elseif(Tp<=Tbub)
	%fprintf( "\n========================== BUBT => T\n" ) ;
	xis      = zis               ;                           % [mol.frac.]
	params   = SRK_mix_parameters_kij( Tp , P , xis , props , alphafun, mixrule ) ;
	Zs       = SRK_compress_factors( params )                                    ;
	Sres     = SRK_Sres( Tp , params , Zs(2) )      ;       % [J/mol]

	dSid = 0;
	for(k=1:Nel)
		dSid=dSid+zis(k)*Sfun{k}(Tp);
	end
	
	dSmol    = dSid - R*log(P) - Sres                 ;       % [J/mol]
else
	%fprintf( "\n========================== DEWT => BUBT\n" ) ;
	bV       = 1        ;                                      % [] vapor fraction
	yis      = zis      ;                                      % [mol.frac.]
	xis      = xdews    ;                                      % [mol.frac.]
	Kis      = yis./xis ;                                      % [ ] init
	prevKis  = Kis      ;                                      % []
	nptslv=4; % Increase the number in case this is not working. 3 seems the minimum
	incT=(Tp-Tdew)/nptslv;
	for( T = Tdew + incT : incT : Tp ) % 
		
		for( k = 1 : maxiter )
			
			for( j = 1 : maxiter )
				
				% ----------------------------------- Newton-Raphson iteration
				OFs = [ ] ;                                    % [] then [OF] then [OF incOF]
				for( bV = [ bV  ( bV + dT ) ] )                % 2 rounds, OF and incOF

					OF = sum( zis.*( Kis - 1 )./( 1 + bV*( Kis - 1 ) ) ) ; % [] Obj. Func.
					OFs   = [ OFs OF ]          ;              % [OF] then [OF incOF]
					
				end
				bV  = bV - dT - OFs( 1 )/( OFs( 2 ) - OFs( 1 ) )*dT ; % [K]
				if( abs( OFs( 1 ) ) < convTtol )
					break ;
				end

			end
			assert( j < maxiter , "\n----- Error: max. iterations reached" ) ;
			
			xis      = zis./( 1 + bV*( Kis - 1 ) )   ;         % [mol.frac]
			yis      = xis.*Kis                      ;         % [mol.frac]

			% ---------------------------------------------------- Soave-improving Kis
			paramsl = SRK_mix_parameters_kij( T , P , xis , props , 'SRK', 'Virial') ;
			Zl      = SRK_compress_factors( paramsl )(2)          ; % []
			phisl   = SRK_parc_fug_coef_all( xis , paramsl , Zl ) ; % []
			
			paramsv = SRK_mix_parameters_kij( T , P , yis , props , 'SRK', 'Virial') ;
			Zv      = SRK_compress_factors( paramsv )(1)          ; % []
			phisv   = SRK_parc_fug_coef_all( yis , paramsv , Zv ) ; % []
			
			fugsv  = phisv.*yis   ;                            % []
			fugsl  = phisl.*xis   ;                            % []
			Kis    = phisl./phisv ;                            % []

			errKis = sum( abs( Kis - prevKis ) )      ;        % []
			if( errKis < convTtol )
				break ;
			end
			prevKis = Kis                        ;             % []
		end
		assert( k < maxiter , "Max. number of iterations reached (state flash)" ) ;
		assert( abs( sum(xis) - 1 ) < 1e-8 , "sum(xis) - 1 > 1e-8" )              ; 
		assert( abs( sum(yis) - 1 ) < 1e-8 , "sum(yis) - 1 > 1e-8" )              ; 
		errFug = sum( ( fugsl./fugsv .- 1 ).^2 )                                  ;
		assert( errFug < 1e-12 , "sum( ( fugsl./fugsv .- 1 ).^2 )  > 1e-12" )     ;      
	end
	Sresl    = SRK_Sres( T , paramsl , Zl )        ;       % [J/mol]
	Sresv    = SRK_Sres( T , paramsv , Zv )        ;       % [J/mol]
	Sres     = ( 1 - bV )*Sresl + bV*Sresv         ;       % [J/mol]

	dSid = 0;
	for(k=1:Nel)
		dSid=dSid+zis(k)*Sfun{k}(T);
	end
	
	dSmol    = dSid - R*log(P) - Sres ;                       % [J/mol]
end
TSpoint    = [ Tp dSmol  ] ;
end